/* $ Jonathan Katz: incrun.g,v 1.1 1997/03/10 23:28:36 jkatz Exp- 
    Adapted by Jason Wittenberg 12/1/01 $ */
external matrix _IncRun_NC, _IncRun_unc, __output,_IncRun_M,_IncRun_Verbose;

proc (11) = incrun(con1,lab1,lib1,con2,lab2,lib2,ninc2,pca);
  local dataset,V1,V2,ConInc,LabInc,LibInc,MP1,unc,uncV,unc2,unc2V,
         V1R1,V1R2,V1R3,V1R4,V1R5,V1R6,X,dta,theta,mlik,grad,cov,
     retcode,N,Y1,NC,dtemp1,tmp,ttw,pcp,mse;
  local iv,ivi,ivc,ivci,ConEff,LabEff,LibEff,ttwev,ttwpv,yhat;    
  
  dta = 0; iv = 0; ivc = 0;
       
  /** Get Ride of Missing Values ***/
 
  dataset = packr(con1~lab1~lib1~con2~lab2~lib2~ninc2~pca);
  V1 = dataset[.,1:3];
  V2 = dataset[.,4:6];
 
  
  /** Useful Values **/
  N = rows(dataset);
  
  /** Create Incumbency Dummy Variables **/
  ConInc = dataset[.,7].==1;
  LabInc = dataset[.,7].==2;
  LibInc = dataset[.,7].==3;
  
  /** Impute Values for uncontested lagged vote **/
  NC = _IncRun_NC;
  MP1 = -9*ones(rows(V1[.,1]),1);
  MP1 = substute(MP1,V1[.,1].<=NC .and V1[.,2].>NC  .and V1[.,3].>NC,1);
  MP1 = substute(MP1,V1[.,1].>NC  .and V1[.,2].<=NC .and V1[.,3].>NC,2);
  MP1 = substute(MP1,V1[.,1].>NC  .and V1[.,2].>NC  .and V1[.,3].<=NC,3);
  MP1 = substute(MP1,V1[.,1].>NC  .and V1[.,2].<=NC .and V1[.,3].<=NC,4);
  MP1 = substute(MP1,V1[.,1].<=NC .and V1[.,2].>NC  .and V1[.,3].<=NC,5);
  MP1 = substute(MP1,V1[.,1].<=NC .and V1[.,2].<=NC .and V1[.,3].>NC,6);

  unc   = _IncRun_unc;
  uncV  =  unc.*ones(N,1);
  unc2  =  unc./2;
  unc2V =  unc2.*ones(N,1);
  
  V1R1 = uncV~(V1[.,2].*(1-unc))~(V1[.,3].*(1-unc));
  V1 = substute(V1,MP1.==1,V1R1);
  
  V1R2 = (V1[.,1].*(1-unc))~uncV~(V1[.,3].*(1-unc));
  V1 = substute(V1,MP1.==2,V1R2);

  V1R3 = (V1[.,1].*(1-unc))~(V1[.,2].*(1-unc))~uncV;
  V1 = substute(V1,MP1.==3,V1R3);

  V1R4 = (1 - uncV)~(unc2V)~(unc2V);
  V1 = substute(V1,MP1.==4,V1R4);
    
  V1R5 = (unc2V)~(1 - uncV)~(unc2V);
  V1 = substute(V1,MP1.==5,V1R5);

  V1R6 = (unc2V)~(unc2V)~(1 - uncV);
  V1 = substute(V1,MP1.==6,V1R6);

  
  /** Convert lagged yote to logit scale **/
  Y1 = ln(V1[.,1:2]./V1[.,3]);
  
  /** Create X's **/
  X = ones(N,1)~Y1~ConInc~LabInc~LibInc;

  /** Pack data **/
  dta = lput(dta,V2[.,1]~X,1);
  dta = lput(dta,V2[.,2]~X,2);
  pca=dataset[.,8];               /* Pull out district ID */
  dta = lput(dta,pca,3);          /* Put ID into estimation datraset */
  

  /** Estimation **/
  if scalone(_IncRun_Verbose);
      __output = 1;
    else;
      __output = 0;
    endif;
    
  _cml_ParNames = { Const_1, YL1_1, YL2_1, ConInc_1, LabInc_1, LibInc_1,
                   Const_2, YL1_2, YL2_2, ConInc_2, LabInc_2, LibInc_2,
           Tdf, TS11, TS12, TS22 };
           
 
           
  { theta,mlik,grad,cov,retcode } = RegMT2(dta);

  /** Simulate Inc Effet **/
  /* Intial setup */
  ivi = { 0,0,0,1,1,1 };
  iv = lput(iv,ivi,1);
  iv = lput(iv,ivi,2);

  /* Conservative */
  ivci =  { 0,0,0,1,0,0 };
  ivc = lput(ivc,ivci,1);
  ivc = lput(ivc,ivci,2);

  /* Return E(V) and E(Y) for each observation 
     from simiulation routine */  
  { ConEff,ttwpv,ttwev,yhat,pcp,mse } = 
           incsim(theta,cov,dta,iv,ivc,_IncRun_M);    
            
  /*   Simulate Labor and Liberal effects
       to get expected values for each observation */ 
  
  /* Labour */
  ivci =  { 0,0,0,0,1,0 };
  ivc = lput(ivc,ivci,1);
  ivc = lput(ivc,ivci,2);
  ttw=0;                 /* Turn off ttw computations */
  
  { LabEff } = incsim2(theta,cov,dta,iv,ivc,_IncRun_M);
       
  /* Lib/Alliance */
  ivci =  { 0,0,0,0,0,1 };
  ivc = lput(ivc,ivci,1);
  ivc = lput(ivc,ivci,2);
  ttw=0; 
  
  { LibEff } = incsim2(theta,cov,dta,iv,ivc,_IncRun_M);  


  retp(theta,cov,coneff,labeff,libeff,
       ttwpv,ttwev,yhat,pcp,mse,dta);
endp;
