/* $ Jonathan Katz: incsim.g,v 1.3 1997/03/10 23:26:36 jkatz Exp $
** Adapted by Jason Wittenberg, 12/20/2001
** witty@polisci.wisc.edu
** 
** Purpose: Calculate Incummbency effects via simulation.
*/

proc (1) = IncSim2(theta,vc,dta,iv,ivc,M);
  clearg k,N,P,ThetaM,X1,X2,k1,iv1,eiv1,XO1,eivc1,XI1,k2,iv2,eiv2,XO2,eivc2,
        XI2,tmarker,j,l,thetaj,b1j,b2j,dfj,Sigmaj,YOj,VOj,YIj,VIj,effj,Eff,Ej,
    ivc1,ivc2,ME;

  Eff = {};
   
  /*** Useful Values ***/
  k = rows(theta);
  N = rows(lget(dta,1));
  P = 2;
  ME = 300;
  
  /*** Draw the M paramaters from multivariate T ***/
  ThetaM = theta + rndmn(zeros(k,1),vc,M)';
  
  /*** Seperate Out X1 and X2 ***/
  X1 = lget(dta,1);
  X1 = X1[.,2:cols(X1)];             /* Original Data for 1            */
  X1 = meanc(X1)';
  k1 = cols(X1);
  iv1 = lget(iv,1)';
  XO1 = substute(X1,iv1,0);         /* Data for 1 assuming Open Seat */
  ivc1 = lget(ivc,1)';
  XI1 = substute(XO1,ivc1,1);       /* Data for 1 assuming Inc Seat  */
  
  X2 = lget(dta,2);
  X2 = X2[.,2:cols(x2)];             /* Original Data for 2            */
  X2 = meanc(X2)';
  k2 = cols(X2);
  iv2 = lget(iv,2)';
  XO2 = substute(X2,iv2,0);         /* Data for 2 assuming Open Seat */
  ivc2 = lget(ivc,2)';
  XI2 = substute(XO2,ivc2,1);       /* Data for 2 assuming Inc Seat  */
  
    /*** Create Marker to pull elements of theta ***/ 
  tmarker = 1*ones(k1,1) | 2*ones(k2,1) | 3 | 4*ones(P*(P+1)/2,1);
  
  /*** Loop for Paramater draws ***/
  for j (1,M,1);
    /* Grab appropriate theta from draws */ 
    thetaj = ThetaM[.,j];
    
    /* Transform the Paramaters */
    b1j = selif(thetaj,tmarker.==1);
    b2j = selif(thetaj,tmarker.==2);
    dfj = tdfi(p,thetaj);
    Sigmaj = tsigi(p,thetaj);
  
    /* Draw Errors */
    Ej = rndmt(zeros(p,1),Sigmaj,dfj,ME);
    Ej = substute(Ej,Ej.>700,700);         /* Cap Ej at 700 to avoid missing values in Pvj */
    Ej = substute(Ej,Ej.<-700,-700);
    Ej = missrv(Ej,0);

    /* Open Seat Predictions */
    YOj =  (XO1*b1j ~ XO2*b2j) + Ej;
    VOj = meanc(mnli(YOj))';
    
    /* Inc Seat Predictions */
    YIj = (XI1*b1j ~ XI2*b2j) + Ej;
    VIj = meanc(mnli(YIj))';

    effj = VIj-VOj;
    Eff = Eff | effj;
  endfor;

  retp(Eff);
endp;
